import math


def split_number_into_chunks(number_to_split, chunk_length):

    _div = math.ceil(number_to_split / chunk_length)

    chunks = [min(chunk_length, number_to_split-(i*chunk_length)) for i in range(_div)]

    return chunks



# Urls
API_URL = 'https://stockx.com/api'
ALGOLIA_URL = (
    "https://xw7sbct9v6-dsn.algolia.net/1/indexes/products/query?x-algolia-agent=Algolia for JavaScript (4.8.6); "
    "Browser "
)

# End points
GET_PRODUCT = 'products'
SEARCH_PRODUCTS = 'browse'
CHART_DATA = 'chart'
SOLD_DATA = 'activity'

# Algolia headers
ALGOLIA_HEADERS = {
    'x-algolia-api-key': 'ZTEwZjZhMzE3NWRiNDRkMTBkYWQ1NjRkMDA2MmI3NGNlYjZkNTNhM2RjZDRmZTI0YmEyZjEzM2Y2MGE4NzAxYnZhbGlkVW50aWw9MTYxNzY1NDEwOA==',
    'x-algolia-application-id': 'XW7SBCT9V6'
}

# Limits
SOLD_DATA_LIMIT = 250

SEARCH_PRODUCTS_OLD_API_PRODUCTS_LIMIT = 20
SEARCH_PRODUCTS_OLD_API_PAGES_LIMIT = 50
SEARCH_PRODUCTS_OLD_API_HITS_LIMIT = SEARCH_PRODUCTS_OLD_API_PRODUCTS_LIMIT*SEARCH_PRODUCTS_OLD_API_PAGES_LIMIT

SEARCH_PRODUCTS_NEW_API_HITS_LIMIT = 1000

# States
SOLD_STATE = 480



from stockx_wrapper import settings as st
from stockx_wrapper.products.product import Product
from stockx_wrapper.requester import requester
from stockx_wrapper.utils import split_number_into_chunks


class Products:

    @staticmethod
    def get_product_data(product_id, country='US', currency='USD'):
        """
        Get product data by product id.

        :param product_id: str
        :param country: str, optional
            Country for focusing market information.
        :param currency: str, optional
            Currency to get. Tested with 'USD' and 'EUR'.

        :return: Product
            Product info.
        """

        # Format url and get data
        url = f'{st.API_URL}/{st.GET_PRODUCT}/{product_id}'
        params = {
            'includes': 'market',
            'currency': currency,
            'country': country
        }
        data = requester.get(url=url, params=params)
        _product = data.get('Product')

        if _product:
            return Product(product_data=_product)

        return None

    def search_products(self, product_name, product_category=None, gender=None, year=None, retail_price=None,
                        shoe_size=None, country='US', currency='USD', market_lowest_ask=None, tags=None,
                        number_of_products=1, more_data=False):
        """
        Search by product name.

        :param product_name: str
        :param product_category: str, optional
            Category of product. For example, 'sneakers'.
        :param gender: str, optional
            Gender of the product. For example, 'women'.
        :param year: int, optional
            Year of release.
        :param retail_price: int, optional
            Filter by retail price. For example,    'lte-100' (less equal than 100),
                                                    'gte-200' (greater equal than 200),
                                                    'range(200|300) (between 200 and 300)
        :param shoe_size: int, optional
            Size of the shoe if a sneaker is asked.
        :param country: str, optional
            Country for focusing market information.
        :param currency: str, optional
            Currency to get. Tested with 'USD' and 'EUR'.
        :param market_lowest_ask: list of str, optional
            Filter by market lowest ask. Follows same schema as retail_price.
        :param tags: list of str, optional
            Tags to focus the search. For example, 'air jordan'.
        :param number_of_products: int, optional
            Number of hits to return.
        :param more_data: bool, optional
            If given, return data will be more exhaustive.

        :return: list of Products
            Product info.
        """

        # Number of hits is limited by default
        products_to_fetch = min(number_of_products, st.SEARCH_PRODUCTS_OLD_API_HITS_LIMIT)

        chunks = split_number_into_chunks(products_to_fetch, st.SEARCH_PRODUCTS_OLD_API_PRODUCTS_LIMIT)

        products = []

        for page, number_to_get in enumerate(chunks):
            # Format url and get data
            url = f'{st.API_URL}/{st.SEARCH_PRODUCTS}'
            params = {
                'page': page+1,
                '_search': product_name,
                '_tags': ','.join(tags) if tags else None,
                'productCategory': product_category,
                'gender': gender,
                'retailPrice': ','.join(retail_price) if retail_price else None,
                'year': year,
                'shoeSize': shoe_size,
                'currency': currency,
                'country': country,
                'market.lowestAsk': ','.join(market_lowest_ask) if market_lowest_ask else None,
                'dataType': 'product' if not product_category else None
            }
            data = requester.get(url=url, params=params)
            _products = data.get('Products')

            if not _products:
                break

            # Return first hit
            products.extend([self.get_product_data(product_id=product_data['id'], country=country, currency=currency)
                             if more_data else
                             Product(product_data=product_data)
                             for product_data in _products[:number_to_get]])

        return products

    def search_products_new_api(self, product_name, number_of_products=1, country='US', currency='USD', more_data=False):
        """
        Uses new API from Algolia.

        :param product_name: str
        :param number_of_products: int, optional
            Number of hits to return.
        :param country: str, optional
            Country for focusing market information.
        :param currency: str, optional
            Currency to get. Tested with 'USD' and 'EUR'.
        :param more_data: bool, optional
            If given, return data will be more exhaustive.

        :return: list of Products
            Product info.
        """

        # Number of hits is limited by default
        products_to_fetch = min(number_of_products, st.SEARCH_PRODUCTS_NEW_API_HITS_LIMIT)

        body = {
            'query': product_name,
            'facets': '*',
            'filters': '',
            "hitsPerPage": products_to_fetch,
        }

        data = requester.post(url=st.ALGOLIA_URL, body=body)
        products = data.get('hits')

        if not products:
            return []

        return [self.get_product_data(product_id=product_data['id'], country=country, currency=currency)
                if more_data else
                Product(product_data=product_data)
                for product_data in products]



import datetime

from stockx_wrapper import settings as st
from stockx_wrapper.requester import requester


class Prices:

    def __init__(self, product_id):
        self.id = product_id

    def get_price_chart_data(self, start_date='all', end_date=datetime.date.today().strftime('%Y-%m-%d'),
                             intervals=100, country='US', currency='USD'):
        """
        Get product price chart. Average price over time.

        :param start_date: str, optional
            Has to be 'all' or 'YYYY-mm-dd' format.
        :param end_date: str, optional
            Has to be 'YYYY-mm-dd' format.
        :param intervals: str, optional
            Number of rows to get. Time between data returned decreases as this param increases.
        :param country: str, optional
            Country for focusing market information.
        :param currency: str, optional
            Currency to get. Tested with 'USD' and 'EUR'.

        :return: list of dicts

        """

        url = f'{st.API_URL}/{st.GET_PRODUCT}/{self.id}/{st.CHART_DATA}'
        params = {
            'start_date': start_date,
            'end_date': end_date,
            'intervals': intervals,
            'country': country,
            'currency': currency
        }
        data = requester.get(url=url, params=params)

        if not data:
            return None

        return_data = []

        for price, date_interval in zip(data['series'][0]['data'], data['xAxis']['categories']):
            return_data.append({
                'avgPrice': price,
                'dateInterval': date_interval
            })

        return return_data

    def get_price_sold_data(self, number_of_items=10, country='US', currency='USD'):
        """
        Get the last {number_of_items} sold prices.

        :param number_of_items: int
            It has to be less than 250 (Stockx API limit). If it's not, ignore it.
        :param country: str, optional
            Country for focusing market information.
        :param currency: str, optional
            Currency to get. Tested with 'USD' and 'EUR'.
        :return:
        """

        _number_of_items = min(number_of_items, st.SOLD_DATA_LIMIT)

        url = f'{st.API_URL}/{st.GET_PRODUCT}/{self.id}/{st.SOLD_DATA}'
        params = {
            'state': st.SOLD_STATE,
            'country': country,
            'currency': currency,
            'limit': _number_of_items,
            'page': 1,
            'sort': 'createdAt',
            'order': 'DESC'
        }

        data = requester.get(url=url, params=params)
        _prices = data.get('ProductActivity')

        return _prices



from stockx_wrapper.products.prices import Prices


class Product:

    def __init__(self, product_data):
        self.parent_id = product_data.get('parentId', None)
        self.id = product_data.get('id', None)
        self.style_id = product_data.get('styleId', None)
        self.name = product_data.get('name', None)
        self.full_name = product_data.get('title', None)
        self.description = product_data.get('description', None)
        self.brand = product_data.get('brand', None)
        self.retail_price = product_data.get('retailPrice', None)
        self.category = product_data.get('productCategory', None)
        self.shoe_size = product_data.get('shoeSize', None)
        self.media = product_data.get('media', None)
        self.market = product_data.get('market', None)
        self.children = product_data.get('children', None)

        self.prices = Prices(product_id=self.id)

    def get_specific_size(self, size):
        """
        Iterate through product children searching for specific size.

        :param size: str
            Size in US format (Ex: 9)

        :return: Product
            Product data.
        """

        if self.children:
            child_data = next((child for child in self.children.values() if child['shoeSize'] == size), None)

            if child_data:
                return Product(product_data=child_data)

        return None
